@@ -19,12 +19,12 @@ void __attribute__((weak))
 {
 }
 
-void *lws_realloc(void *ptr, size_t size)
+void *lws_realloc(void *ptr, size_t size, const char *reason)
 {
 	return TEE_Realloc(ptr, size);
 }
 
-void *lws_malloc(size_t size)
+void *lws_malloc(size_t size, const char *reason)
 {
 	return TEE_Malloc(size, TEE_USER_MEM_HINT_NO_FILL_ZERO);
 }
@@ -34,7 +34,7 @@ void lws_free(void *p)
 	TEE_Free(p);
 }
 
-void *lws_zalloc(size_t size)
+void *lws_zalloc(size_t size, const char *reason)
 {
 	void *ptr = TEE_Malloc(size, TEE_USER_MEM_HINT_NO_FILL_ZERO);
 	if (ptr)
@@ -42,41 +42,48 @@ void *lws_zalloc(size_t size)
 	return ptr;
 }
 
-void lws_set_allocator(void *(*cb)(void *ptr, size_t size))
+void lws_set_allocator(void *(*cb)(void *ptr, size_t size, const char *reason))
 {
 	(void)cb;
 }
 #else
 
-static void *_realloc(void *ptr, size_t size)
+static void *_realloc(void *ptr, size_t size, const char *reason)
 {
-	if (size)
+	if (size) {
+#if defined(LWS_PLAT_ESP32)
+		lwsl_notice("%s: size %lu: %s\n", __func__, (unsigned long)size, reason);
+#else
+		lwsl_debug("%s: size %lu: %s\n", __func__, (unsigned long)size, reason);
+#endif
 #if defined(LWS_PLAT_OPTEE)
 		return (void *)TEE_Realloc(ptr, size);
 #else
 		return (void *)realloc(ptr, size);
 #endif
-	else if (ptr)
+	}
+	if (ptr)
 		free(ptr);
+
 	return NULL;
 }
 
-void *(*_lws_realloc)(void *ptr, size_t size) = _realloc;
+void *(*_lws_realloc)(void *ptr, size_t size, const char *reason) = _realloc;
 
-void *lws_realloc(void *ptr, size_t size)
+void *lws_realloc(void *ptr, size_t size, const char *reason)
 {
-	return _lws_realloc(ptr, size);
+	return _lws_realloc(ptr, size, reason);
 }
 
-void *lws_zalloc(size_t size)
+void *lws_zalloc(size_t size, const char *reason)
 {
-	void *ptr = _lws_realloc(NULL, size);
+	void *ptr = _lws_realloc(NULL, size, reason);
 	if (ptr)
 		memset(ptr, 0, size);
 	return ptr;
 }
 
-void lws_set_allocator(void *(*cb)(void *ptr, size_t size))
+void lws_set_allocator(void *(*cb)(void *ptr, size_t size, const char *reason))
 {
 	_lws_realloc = cb;
 }
